# Predefined colors. 
import numpy as np

class Colors:
    """
    Convenience class pre-defining some colors to use with our mesh objects.

    Attributes:
    red (numpy.array): Color red.
    blue (numpy.array): Color blue.
    green (numpy.array): Color green.
    white (numpy.array): Color white.
    black (numpy.array): Color black.
    yellow (numpy.array): Color yellow.
    orange (numpy.array): Color orange.
    purple (numpy.array): Color purple.
    pink (numpy.array): Color pink.
    """
    red = np.array([1,0,0,0])
    blue = np.array([0,0,1,0])
    green = np.array([0,1,0,0])
    white = np.array([1,1,1,0])
    black = np.array([0,0,0,0])
    yellow = np.array([1,1,0,0])
    orange = np.array([1,0.5,0,0])
    purple = np.array([0.8,0,1,0])
    pink = np.array([1,0.8,0.945,0])
